﻿function setActorDirection(actor, direction)
{
    actor.setAttribute("direction", direction);
    if (direction=="south") {
        actor.setCurrentFrame(1);
    }
    else if (direction=="east") {
        actor.setCurrentFrame(5);
    }
    else if (direction=="north") {
        actor.setCurrentFrame(9);
    }
    else if (direction=="west") {
        actor.setCurrentFrame(13);
    }
}

function loadEnvironmentFromKumRobotFile(fileName)
{
    
    // Инициализация гадов
    
    var actors = G.actorNames();
    var aliens = []
    
    for (var i=0; i<actors.length; i++) {
        if (actors[i]!="robot" && actors[i]!="bullet") {
            var alien = G.actor(actors[i]);
            alien.enabled = false;
            alien.visible = false;
            aliens.push(alien);
        }
    }

    // Робот и сцена
    var robot = G.actor("robot");
    var Scene = G.Scene;

    
    // Парсировка .fil-файла
    
    var data = G.readText(fileName, "UTF-8");
    var w, h;
    var actorX, actorY;
    var actorDirection = "south";
    var sizeRead = false;
    var actorPositionRead = false;
    var lines = data.split("\n");
    var cells = [];   
    var aliensPos = [];
     
    for (var lineNo=0; lineNo<lines.length; lineNo++) {
        var line = lines[lineNo];
        if (line.length==0 || line[0]==';')
            continue;
        if (!sizeRead) {
            var values = line.split(" ");
            w = Number(values[0]);
            h = Number(values[1]);
            sizeRead = true;
        }
        else if (!actorPositionRead) {
            var values = line.split(" ");
            actorX = Number(values[0]);
            actorY = Number(values[1]);
            if (values.length>2)
                actorDirection = values[2];
            actorPositionRead = true;
        }
        else {
            var values = line.split(" ");
            var wall = Number(values[2]);              
            var cell = {
                x: Number(values[0]),
                y: h - 1 - Number(values[1]),
                state: Number(values[3])>0? 3 : 0,
                wallLeft: (wall & 0x1) > 0? 1 : 0,
                wallRight: (wall & 0x2) > 0? 1 : 0,
                wallBack: (wall & 0x4) > 0? 1 : 0,
                wallFront: (wall & 0x8) > 0? 1 : 0
            };
            if (values.length > 8 && aliensPos.length < 4 && values[8]=="1") {
                aliensPos.push({ x: cell.x, y: cell.y});
            }
            cells.push(cell);
        }
    }
    
    // Инициализация сцены
    
    Scene.setSize(w, h, 1);
    for (var y=0; y<h; y++) {
        for (var x=0; x<w; x++) {
            var cell = {
                x: x,
                y: y,
                state: 0,
                wallLeft: 0,
                wallRight: 0,
                wallBack: 0,
                wallFront: 0
            };
            for (var i=0; i<cells.length; i++) {
                if (cells[i].x==x && cells[i].y==y) {
                    cell = cells[i];
                }
            }
            Scene.setCellAttribute(x,y,0,"painted",cell.state);
            Scene.setCellBorderState(x,y,0,"bottom",cell.state);
            if (cell.wallLeft>0)
                Scene.setCellBorderState(x,y,0,"left",cell.wallLeft);
            if (cell.wallRight>0)
                Scene.setCellBorderState(x,y,0,"right",cell.wallRight);
            if (cell.wallFront>0)
                Scene.setCellBorderState(x,y,0,"front",cell.wallFront);
            if (cell.wallBack>0)
                Scene.setCellBorderState(x,y,0,"back",cell.wallBack);
        }
    }
    
    // Стены вокруг всей сцены
    
    
    for (var y=0; y<h; y++) {
        Scene.setCellBorderState(0,y,0,"left",2);
        Scene.setCellBorderState(w-1,y,0,"right",1);
    }
    
    for (var x=0; x<w; x++) {
        Scene.setCellBorderState(x,0,0,"back",1);
        Scene.setCellBorderState(x,h-1,0,"front",2);
    }
    
    
        
    // Инициализация актора
    
    if (robot!=null) { 
        robot.x = actorX;
        robot.y = Scene.length - 1 - actorY;
        robot.z = 0;
        var revertedDirection;
        // инвертируем положение робота, т.к. 
        // у протоисполнителя нулевая точка отсчета -- это 
        // левый нижний угол, а у обычного робота --
        // левый верхний
        if (actorDirection=="south")
            revertedDirection = "north";
        else if (actorDirection=="north")
            revertedDirection = "south";
        else
            revertedDirection = actorDirection;
        setActorDirection(robot, actorDirection);
        robot.visible = true;
    }
    
    // Расставляем гадов
    
    var aliensCount = aliensPos.length;
    if (aliens.length < aliensCount)
        aliensCount = aliens.length;
    
    
    for (var i=0; i<aliensCount; i++) {
        aliens[i].x = aliensPos[i].x;
        aliens[i].y = aliensPos[i].y;
        aliens[i].z = 0;
        setActorDirection(aliens[i], "south");
        aliens[i].visible = true;
        aliens[i].enabled = true;
        aliens[i].setAttribute("rotating", 0);
    }
    
}

